@@ -4,6 +4,9 @@ import os
 import json
 import asyncio
 import base64
+import random
+import string
+import urllib.parse
 from typing import AsyncIterator
 from urllib.parse import quote
 
@@ -25,11 +28,12 @@ from ..typing import AsyncResult, Messages, MediaListType
 from ..errors import MissingRequirementsError, NoValidHarFileError, MissingAuthError
 from ..providers.response import *
 from ..tools.media import merge_media
-from ..requests import get_nodriver
+from ..requests import get_nodriver, DEFAULT_HEADERS
 from ..image import to_bytes, is_accepted_format
 from .helper import get_last_user_message
 from ..files import get_bucket_dir
 from ..tools.files import read_bucket
+from ..cookies import get_cookies
 from pathlib import Path
 from .. import debug
 
@@ -51,34 +55,67 @@ def extract_bucket_items(messages: Messages) -> list[dict]:
             bucket_items = []
     return bucket_items
 
+def random_hex(length):
+    return ''.join(random.choices('0123456789ABCDEF', k=length))
+
+def random_base64(length):
+    chars = string.ascii_letters + string.digits + '+/='
+    return ''.join(random.choices(chars, k=length))
+
+def get_fake_cookie():
+    return {
+        "_C_ETH": "1",
+        "_C_Auth": "",
+        "MUID": random_hex(32),
+        "MUIDB": random_hex(32),
+        "_EDGE_S": f"F=1&SID={random_hex(32)}",
+        "_EDGE_V": "1",
+        "ak_bmsc": f"{random_hex(32)}~{'0'*48}~{urllib.parse.quote(random_base64(300))}"
+    }
+
 class Copilot(AsyncAuthedProvider, ProviderModelMixin):
     label = "Microsoft Copilot"
     url = "https://copilot.microsoft.com"
+    cookie_domain = ".microsoft.com"
+    anon_cookie_name = "__Host-copilot-anon"
     
     working = True
-    supports_stream = True
+    use_nodriver = has_nodriver
     active_by_default = True
     
     default_model = "Copilot"
-    models = [default_model, "Think Deeper", "Smart (GPT-5)"]
+    models = [default_model, "Think Deeper", "Smart (GPT-5)", "Study"]
     model_aliases = {
         "o1": "Think Deeper",
         "gpt-4": default_model,
         "gpt-4o": default_model,
         "gpt-5": "GPT-5",
+        "study": "Study",
     }
 
     websocket_url = "wss://copilot.microsoft.com/c/api/chat?api-version=2"
     conversation_url = f"{url}/c/api/conversations"
 
-    _access_token: str = None
-    _cookies: dict = {}
-
     @classmethod
-    async def on_auth_async(cls, **kwargs) -> AsyncIterator:
+    async def on_auth_async(cls, cookies: dict = None, proxy: str = None, **kwargs) -> AsyncIterator:
+        if cookies is None:
+            cookies = get_fake_cookie() or get_cookies(cls.cookie_domain, False, cache_result=False)
+        access_token = None
+        useridentitytype = None
+        if cls.needs_auth or cls.anon_cookie_name not in cookies:
+            try:
+                access_token, useridentitytype, cookies = readHAR(cls.url)
+            except NoValidHarFileError as h:
+                debug.log(f"Copilot: {h}")
+                if has_nodriver:
+                    yield RequestLogin(cls.label, os.environ.get("G4F_LOGIN_URL", ""))
+                    access_token, useridentitytype, cookies = await get_access_token_and_cookies(cls.url, proxy, cls.needs_auth)
+                else:
+                    raise h
         yield AuthResult(
-            api_key=cls._access_token,
-            cookies=cls.cookies_to_dict()
+            access_token=access_token,
+            useridentitytype=useridentitytype,
+            cookies=cookies
         )
 
     @classmethod
@@ -87,88 +124,95 @@ class Copilot(AsyncAuthedProvider, ProviderModelMixin):
         model: str,
         messages: Messages,
         auth_result: AuthResult,
-        **kwargs
-    ) -> AsyncResult:
-        cls._access_token = getattr(auth_result, "api_key")
-        cls._cookies = getattr(auth_result, "cookies")
-        async for chunk in cls.create(model, messages, **kwargs):
-            yield chunk
-        auth_result.cookies = cls.cookies_to_dict()
-
-    @classmethod
-    def cookies_to_dict(cls):
-        return cls._cookies if isinstance(cls._cookies, dict) else {c.name: c.value for c in cls._cookies}
-
-    @classmethod
-    async def create(
-        cls,
-        model: str,
-        messages: Messages,
         proxy: str = None,
         timeout: int = 30,
         prompt: str = None,
         media: MediaListType = None,
         conversation: BaseConversation = None,
         return_conversation: bool = True,
-        useridentitytype: str = "google",
-        api_key: str = None,
         **kwargs
     ) -> AsyncResult:
         if not has_curl_cffi:
             raise MissingRequirementsError('Install or update "curl_cffi" package | pip install -U curl_cffi')
         model = cls.get_model(model)
         websocket_url = cls.websocket_url
-        headers = None
-        if cls._access_token or cls.needs_auth:
-            if api_key is not None:
-                cls._access_token = api_key
-            if cls._access_token is None:
-                try:
-                    cls._access_token, cls._cookies = readHAR(cls.url)
-                except NoValidHarFileError as h:
-                    debug.log(f"Copilot: {h}")
-                    if has_nodriver:
-                        yield RequestLogin(cls.label, os.environ.get("G4F_LOGIN_URL", ""))
-                        cls._access_token, cls._cookies = await get_access_token_and_cookies(cls.url, proxy)
-                    else:
-                        raise h
-            websocket_url = f"{websocket_url}&accessToken={quote(cls._access_token)}&X-UserIdentityType={quote(useridentitytype)}"
-            headers = {"authorization": f"Bearer {cls._access_token}"}
+        headers = DEFAULT_HEADERS.copy()
+        headers["origin"] = cls.url
+        headers["referer"] = cls.url + "/"
+        if getattr(auth_result, "access_token", None):
+            websocket_url = f"{websocket_url}&accessToken={quote(auth_result.access_token)}" + (f"&X-UserIdentityType={quote(auth_result.useridentitytype)}" if getattr(auth_result, "useridentitytype", None) else "")
+            headers["authorization"] = f"Bearer {auth_result.access_token}"
 
         async with AsyncSession(
             timeout=timeout,
             proxy=proxy,
             impersonate="chrome",
             headers=headers,
-            cookies=cls._cookies,
+            cookies=auth_result.cookies
         ) as session:
-            if cls._access_token is not None:
-                cls._cookies = session.cookies.jar if hasattr(session.cookies, "jar") else session.cookies
-                response = await session.get("https://copilot.microsoft.com/c/api/user?api-version=2", headers={"x-useridentitytype": useridentitytype})
-                if response.status_code == 401:
-                    raise MissingAuthError("Status 401: Invalid access token")
-                response.raise_for_status()
-                user = response.json().get('firstName')
-                if user is None:
-                    if cls.needs_auth:
-                        raise MissingAuthError("No user found, please login first")
-                    cls._access_token = None
-                else:
-                    debug.log(f"Copilot: User: {user}")
             if conversation is None:
-                response = await session.post(cls.conversation_url, headers={"x-useridentitytype": useridentitytype} if cls._access_token else {})
+                # har_file = os.path.join(os.path.dirname(__file__), "copilot", "copilot.microsoft.com.har")
+                # with open(har_file, "r") as f:
+                #     har_entries = json.load(f).get("log", {}).get("entries", [])
+                # conversationId = ""
+                # for har_entry in har_entries:
+                #     if har_entry.get("request"):
+                #         if "/c/api/" in har_entry.get("request").get("url", ""):
+                #             try:
+                #                 response = await getattr(session, har_entry.get("request").get("method").lower())(
+                #                     har_entry.get("request").get("url", "").replace("cvqBJw7kyPAp1RoMTmzC6", conversationId),
+                #                     data=har_entry.get("request").get("postData", {}).get("text"),
+                #                     headers={header["name"]: header["value"] for header in har_entry.get("request").get("headers")}
+                #                 )
+                #                 response.raise_for_status()
+                #                 if response.headers.get("content-type", "").startswith("application/json"):
+                #                     conversationId = response.json().get("currentConversationId", conversationId)
+                #             except Exception as e:
+                #                 debug.log(f"Copilot: Failed request to {har_entry.get('request').get('url', '')}: {e}")
+                data = {
+                    "timeZone": "America/Los_Angeles",
+                    "startNewConversation": True,
+                    "teenSupportEnabled": True,
+                    "correctPersonalizationSetting": True,
+                    "performUserMerge": True,
+                    "deferredDataUseCapable": True
+                }
+                response = await session.post(
+                    "https://copilot.microsoft.com/c/api/start",
+                    headers={
+                        "content-type": "application/json",
+                        **({"x-useridentitytype": auth_result.useridentitytype} if getattr(auth_result, "useridentitytype", None) else {}),
+                        **(headers or {})
+                    },
+                    json=data
+                )
+                if response.status_code == 401:
+                    raise MissingAuthError("Status 401: Invalid session")
                 response.raise_for_status()
-                conversation_id = response.json().get("id")
-                conversation = Conversation(conversation_id)
-                debug.log(f"Copilot: Created conversation: {conversation_id}")
+                debug.log(f"Copilot: Update cookies: [{', '.join(key for key in response.cookies)}]")
+                auth_result.cookies.update({key: value for key, value in response.cookies.items()})
+                if not cls.needs_auth and cls.anon_cookie_name not in auth_result.cookies:
+                    raise MissingAuthError(f"Missing cookie: {cls.anon_cookie_name}")
+                conversation = Conversation(response.json().get("currentConversationId"))
+                debug.log(f"Copilot: Created conversation: {conversation.conversation_id}")
             else:
-                conversation_id = conversation.conversation_id
-                debug.log(f"Copilot: Use conversation: {conversation_id}")
-            if return_conversation:
-                yield conversation
+                debug.log(f"Copilot: Use conversation: {conversation.conversation_id}")
+
+            # response = await session.get("https://copilot.microsoft.com/c/api/user?api-version=4", headers={"x-useridentitytype": useridentitytype} if cls._access_token else {})
+            # if response.status_code == 401:
+            #     raise MissingAuthError("Status 401: Invalid session")
+            # response.raise_for_status()
+            # print(response.json())
+            # user = response.json().get('firstName')
+            # if user is None:
+            #     if cls.needs_auth:
+            #         raise MissingAuthError("No user found, please login first")
+            #     cls._access_token = None
+            # else:
+            #     debug.log(f"Copilot: User: {user}")
 
             uploaded_attachments = []
-            if cls._access_token is not None:
+            if auth_result.access_token:
                 # Upload regular media (images)
                 for media, _ in merge_media(media, messages):
                     if not isinstance(media, str):
@@ -178,7 +222,7 @@ class Copilot(AsyncAuthedProvider, ProviderModelMixin):
                             headers={
                                 "content-type": is_accepted_format(data),
                                 "content-length": str(len(data)),
-                                **({"x-useridentitytype": useridentitytype} if cls._access_token else {})
+                                **({"x-useridentitytype": auth_result.useridentitytype} if getattr(auth_result, "useridentitytype", None) else {})
                             },
                             data=data
                         )
@@ -201,7 +245,7 @@ class Copilot(AsyncAuthedProvider, ProviderModelMixin):
                                 response = await session.post(
                                     "https://copilot.microsoft.com/c/api/attachments",
                                     multipart=data,
-                                    headers={"x-useridentitytype": useridentitytype}
+                                    headers={"x-useridentitytype": auth_result.useridentitytype} if getattr(auth_result, "useridentitytype", None) else {}
                                 )
                                 response.raise_for_status()
                                 data = response.json()
@@ -225,7 +269,7 @@ class Copilot(AsyncAuthedProvider, ProviderModelMixin):
                 mode = "chat"
             await wss.send(json.dumps({
                 "event": "send",
-                "conversationId": conversation_id,
+                "conversationId": conversation.conversation_id,
                 "content": [*uploaded_attachments, {
                     "type": "text",
                     "text": prompt,
@@ -240,7 +284,7 @@ class Copilot(AsyncAuthedProvider, ProviderModelMixin):
             sources = {}
             while not wss.closed:
                 try:
-                    msg_txt, _ = await asyncio.wait_for(wss.recv(), 3 if done else timeout)
+                    msg_txt, _ = await asyncio.wait_for(wss.recv(), 1 if done else timeout)
                     msg = json.loads(msg_txt)
                 except:
                     break
@@ -274,17 +318,20 @@ class Copilot(AsyncAuthedProvider, ProviderModelMixin):
                 elif msg.get("event") not in ["received", "startMessage", "partCompleted", "connected"]:
                     debug.log(f"Copilot Message: {msg_txt[:100]}...")
             if not done:
-                raise RuntimeError(f"Invalid response: {last_msg}")
+                raise MissingAuthError(f"Invalid response: {last_msg}")
+            if return_conversation:
+                yield conversation
             if sources:
                 yield Sources(sources.values())
             if not wss.closed:
                 await wss.close()
 
-async def get_access_token_and_cookies(url: str, proxy: str = None):
+async def get_access_token_and_cookies(url: str, proxy: str = None, needs_auth: bool = False):
     browser, stop_browser = await get_nodriver(proxy=proxy)
     try:
         page = await browser.get(url)
         access_token = None
+        useridentitytype = None
         while access_token is None:
             for _ in range(2):
                 await asyncio.sleep(3)
@@ -292,9 +339,12 @@ async def get_access_token_and_cookies(url: str, proxy: str = None):
                     (() => {
                         for (var i = 0; i < localStorage.length; i++) {
                             try {
-                                item = JSON.parse(localStorage.getItem(localStorage.key(i)));
+                                const key = localStorage.key(i);
+                                const item = JSON.parse(localStorage.getItem(key));
                                 if (item?.body?.access_token) {
-                                    return item.body.access_token;
+                                    return ["" + item?.body?.access_token, "google"];
+                                } else if (key.includes("chatai")) {
+                                    return "" + item.secret;
                                 }
                             } catch(e) {}
                         }
@@ -302,16 +352,42 @@ async def get_access_token_and_cookies(url: str, proxy: str = None):
                 """)
                 if access_token is None:
                     await asyncio.sleep(1)
+                    continue
+                if isinstance(access_token, list):
+                    access_token, useridentitytype = access_token
+                access_token = access_token.get("value") if isinstance(access_token, dict) else access_token
+                useridentitytype = useridentitytype.get("value") if isinstance(useridentitytype, dict) else None
+                debug.log(f"Got access token: {access_token[:10]}..., useridentitytype: {useridentitytype}")
+                break
+            if not needs_auth:
+                break
+        if not needs_auth:
+            textarea = await page.select("textarea")
+            if textarea is not None:
+                await textarea.send_keys("Hello")
+                await asyncio.sleep(1)
+                button = await page.select("[data-testid=\"submit-button\"]")
+                if button:
+                    await button.click()
+                    turnstile = await page.select('#cf-turnstile')
+                    if turnstile:
+                        debug.log("Found Element: 'cf-turnstile'")
+                        await asyncio.sleep(3)
+                        await click_trunstile(page)
         cookies = {}
-        for c in await page.send(nodriver.cdp.network.get_cookies([url])):
-            cookies[c.name] = c.value
+        while Copilot.anon_cookie_name not in cookies:
+            await asyncio.sleep(2)
+            cookies = {c.name: c.value for c in await page.send(nodriver.cdp.network.get_cookies([url]))}
+            if not needs_auth and Copilot.anon_cookie_name in cookies:
+                break
         stop_browser()
-        return access_token, cookies
+        return access_token, useridentitytype, cookies
     finally:
         stop_browser()
 
 def readHAR(url: str):
     api_key = None
+    useridentitytype = None
     cookies = None
     for path in get_har_files():
         with open(path, 'rb') as file:
@@ -325,9 +401,27 @@ def readHAR(url: str):
                     v_headers = get_headers(v)
                     if "authorization" in v_headers:
                         api_key = v_headers["authorization"].split(maxsplit=1).pop()
+                    if "x-useridentitytype" in v_headers:
+                        useridentitytype = v_headers["x-useridentitytype"]
                     if v['request']['cookies']:
                         cookies = {c['name']: c['value'] for c in v['request']['cookies']}
-    if api_key is None:
-        raise NoValidHarFileError("No access token found in .har files")
+    if not cookies:
+        raise NoValidHarFileError("No session found in .har files")
 
-    return api_key, cookies
\ No newline at end of file
+    return api_key, useridentitytype, cookies
+
+if has_nodriver:
+    async def click_trunstile(page: nodriver.Tab, element='document.getElementById("cf-turnstile")'):
+        for _ in range(3):
+            size = None
+            for idx in range(15):
+                size = await page.js_dumps(f'{element}?.getBoundingClientRect()||{{}}')
+                debug.log(f"Found size: {size.get('x'), size.get('y')}")
+                if "x" not in size:
+                    break
+                await page.flash_point(size.get("x") + idx * 3, size.get("y") + idx * 3)
+                await page.mouse_click(size.get("x") + idx * 3, size.get("y") + idx * 3)
+                await asyncio.sleep(2)
+            if "x" not in size:
+                break
+        debug.log("Finished clicking trunstile.")
\ No newline at end of file
