@@ -91,10 +91,6 @@ class Gemini(AsyncGeneratorProvider, ProviderModelMixin):
     models = [
         default_model, "gemini-2.5-flash", "gemini-2.5-pro"
     ]
-    model_aliases = {
-        "gemini-2.0": "",
-        "gemini-2.0-flash": ["gemini-2.0-flash", "gemini-2.0-flash", "gemini-2.0-flash-exp"],
-    }
 
     synthesize_content_type = "audio/vnd.wav"
     
@@ -106,29 +102,6 @@ class Gemini(AsyncGeneratorProvider, ProviderModelMixin):
     refresh_interval = 540
     rotate_tasks = {}
 
-    @classmethod
-    def get_model(cls, model: str) -> str:
-        """Get the internal model name from the user-provided model name."""
-        if not model:
-            return cls.default_model
-        
-        # Check if the model exists directly in our models list
-        if model in cls.models:
-            return model
-        
-        # Check if there's an alias for this model
-        if model in cls.model_aliases:
-            alias = cls.model_aliases[model]
-            # If the alias is a list, randomly select one of the options
-            if isinstance(alias, list):
-                selected_model = random.choice(alias)
-                debug.log(f"Gemini: Selected model '{selected_model}' from alias '{model}'")
-                return selected_model
-            debug.log(f"Gemini: Using model '{alias}' for alias '{model}'")
-            return alias
-        
-        raise ModelNotFoundError(f"Model {model} not found")
-
     @classmethod
     async def nodriver_login(cls, proxy: str = None) -> AsyncIterator[str]:
         if not has_nodriver:
