@@ -21,10 +21,10 @@ from ..base_provider import AsyncAuthedProvider, ProviderModelMixin
 from ...typing import AsyncResult, Messages, Cookies, MediaListType
 from ...requests.raise_for_status import raise_for_status
 from ...requests import StreamSession
-from ...requests import get_nodriver
+from ...requests import get_nodriver_session
 from ...image import ImageRequest, to_image, to_bytes, is_accepted_format, detect_file_type
 from ...errors import MissingAuthError, NoValidHarFileError, ModelNotFoundError
-from ...providers.response import JsonConversation, FinishReason, SynthesizeData, AuthResult, ImageResponse, ImagePreview, ResponseType, format_link
+from ...providers.response import JsonConversation, FinishReason, SynthesizeData, AuthResult, ImageResponse, ImagePreview, ResponseType, JsonRequest, format_link
 from ...providers.response import TitleGeneration, RequestLogin, Reasoning
 from ...tools.media import merge_media
 from ..helper import format_cookies, format_media_prompt, to_string
@@ -283,7 +283,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
         return messages
 
     @classmethod
-    async def get_generated_image(cls, session: StreamSession, auth_result: AuthResult, element: Union[dict, str], prompt: str = None, conversation_id: str = None) -> AsyncIterator:
+    async def get_generated_image(cls, session: StreamSession, auth_result: AuthResult, element: Union[dict, str], prompt: str = None, conversation_id: str = None) -> ImagePreview|ImageResponse|None:
         download_urls = []
         is_sediment = False
         if prompt is None:
@@ -319,7 +319,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
             debug.error("OpenaiChat: Download image failed")
             debug.error(e)
         if download_urls:
-            return ImagePreview(download_urls, prompt) if is_sediment else ImageResponse(download_urls, prompt)
+            return ImagePreview(download_urls, prompt, {"headers":auth_result.headers}) if is_sediment else ImageResponse(download_urls, prompt, {"headers":auth_result.headers})
 
     @classmethod
     async def create_authed(
@@ -330,14 +330,15 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
         proxy: str = None,
         timeout: int = 360,
         auto_continue: bool = False,
-        action: str = "next",
+        action: Optional[str] = None,
         conversation: Conversation = None,
         media: MediaListType = None,
         return_conversation: bool = True,
         web_search: bool = False,
         prompt: str = None,
-        conversation_mode=None,
-        temporary=False,
+        conversation_mode: Optional[dict] = None,
+        temporary: Optional[bool] = None,
+        conversation_id: Optional[str] = None,
         **kwargs
     ) -> AsyncResult:
         """
@@ -351,7 +352,6 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
             api_key (str): Access token for authentication.
             auto_continue (bool): Flag to automatically continue the conversation.
             action (str): Type of action ('next', 'continue', 'variant').
-            conversation_id (str): ID of the conversation.
             media (MediaListType): Images to include in the conversation.
             return_conversation (bool): Flag to include response fields in the output.
             **kwargs: Additional keyword arguments.
@@ -362,6 +362,10 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
         Raises:
             RuntimeError: If an error occurs during processing.
         """
+        if temporary is None:
+            temporary = action is not None and conversation_id is not None
+        if action is None:
+            action = "next"
         async with StreamSession(
             proxy=proxy,
             impersonate="chrome",
@@ -431,7 +435,6 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
                     }
                     if temporary:
                         data["history_and_training_disabled"] = True
-
                     async with session.post(
                         prepare_url,
                         json=data,
@@ -494,7 +497,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
                 if temporary:
                     data["history_and_training_disabled"] = True
 
-                if conversation.conversation_id is not None:
+                if conversation.conversation_id is not None and not temporary:
                     data["conversation_id"] = conversation.conversation_id
                     debug.log(f"OpenaiChat: Use conversation: {conversation.conversation_id}")
                 prompt = conversation.prompt = format_media_prompt(messages, prompt)
@@ -510,6 +513,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
                             else:
                                 new_messages.append(message)
                     data["messages"] = cls.create_messages(new_messages, image_requests, ["search"] if web_search else None)
+                yield JsonRequest.from_dict(data)
                 headers = {
                     **cls._headers,
                     "accept": "text/event-stream",
@@ -531,7 +535,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
                     headers=headers
                 ) as response:
                     cls._update_request_args(auth_result, session)
-                    if response.status in (401, 403, 429):
+                    if response.status in (401, 403, 429, 500):
                         raise MissingAuthError("Access token is not valid")
                     elif response.status == 422:
                         raise RuntimeError((await response.json()), data)
@@ -643,7 +647,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
                 if sources.list:
                     yield sources
                 if conversation.generated_images:
-                    yield ImageResponse(conversation.generated_images.urls, conversation.prompt)
+                    yield ImageResponse(conversation.generated_images.urls, conversation.prompt, {"headers":auth_result.headers})
                     conversation.generated_images = None
                 conversation.prompt = None
                 if return_conversation:
@@ -852,8 +856,7 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
 
     @classmethod
     async def nodriver_auth(cls, proxy: str = None):
-        browser, stop_browser = await get_nodriver(proxy=proxy)
-        try:
+        async with get_nodriver_session(proxy=proxy) as browser:
             page = await browser.get(cls.url)
             def on_request(event: nodriver.cdp.network.RequestWillBeSent, page=None):
                 if event.request.url == start_url or event.request.url.startswith(conversation_url):
@@ -882,14 +885,21 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
             page.add_handler(nodriver.cdp.network.RequestWillBeSent, on_request)
             await page.reload()
             user_agent = await page.evaluate("window.navigator.userAgent", return_by_value=True)
-            if cls.needs_auth:
-                await page.select('[data-testid="accounts-profile-button"]', 300)
-            textarea = await page.select("#prompt-textarea", 300)
-            await textarea.send_keys("Hello")
-            await asyncio.sleep(1)
+            debug.log(f"OpenaiChat: User-Agent: {user_agent}")
+            for _ in range(3):
+                try:
+                    if cls.needs_auth:
+                        await page.select('[data-testid="accounts-profile-button"]', 300)
+                    textarea = await page.select("#prompt-textarea", 300)
+                    await textarea.send_keys("Hello")
+                    await asyncio.sleep(1)
+                except nodriver.core.connection.ProtocolException:
+                    continue
+                break
             button = await page.select("[data-testid=\"send-button\"]")
             if button:
                 await button.click()
+            debug.log("OpenaiChat: 'Hello' sended")
             while True:
                 body = await page.evaluate("JSON.stringify(window.__remixContext)", return_by_value=True)
                 if hasattr(body, "value"):
@@ -902,17 +912,19 @@ class OpenaiChat(AsyncAuthedProvider, ProviderModelMixin):
                 if cls._api_key is not None or not cls.needs_auth:
                     break
                 await asyncio.sleep(1)
+            debug.log(f"OpenaiChat: Access token: {'False' if cls._api_key is None else cls._api_key[:12]+'...'}")
             while True:
                 if cls.request_config.proof_token:
                     break
                 await asyncio.sleep(1)
+            debug.log(f"OpenaiChat: Proof token: Yes")
             cls.request_config.data_build = await page.evaluate("document.documentElement.getAttribute('data-build')")
             cls.request_config.cookies = await page.send(get_cookies([cls.url]))
             await page.close()
             cls._create_request_args(cls.request_config.cookies, cls.request_config.headers, user_agent=user_agent)
             cls._set_api_key(cls._api_key)
-        finally:
-            stop_browser()
+            debug.log(f"OpenaiChat: Sleep 10s")
+            await asyncio.sleep(10)
 
     @staticmethod
     def get_default_headers() -> Dict[str, str]:
