@@ -8,7 +8,7 @@ from ...typing import Union, AsyncResult, Messages, MediaListType
 from ...requests import StreamSession, StreamResponse, raise_for_status, sse_stream
 from ...image import use_aspect_ratio
 from ...image.copy_images import save_response_media
-from ...providers.response import FinishReason, ToolCalls, Usage, ImageResponse, ProviderInfo, AudioResponse, Reasoning, JsonConversation
+from ...providers.response import *
 from ...tools.media import render_messages
 from ...tools.run_tools import AuthManager
 from ...errors import MissingAuthError
@@ -31,10 +31,9 @@ class OpenaiTemplate(AsyncGeneratorProvider, ProviderModelMixin, RaiseErrorMixin
     max_tokens: int = None
 
     @classmethod
-    def get_models(cls, api_key: str = None, api_base: str = None) -> list[str]:
+    def get_models(cls, api_key: str = None, api_base: str = None, timeout: int = None) -> list[str]:
         if not cls.models:
             try:
-                headers = {}
                 if api_base is None:
                     api_base = cls.api_base
                 if api_key is None and cls.api_key is not None:
@@ -43,12 +42,10 @@ class OpenaiTemplate(AsyncGeneratorProvider, ProviderModelMixin, RaiseErrorMixin
                     api_key = AuthManager.load_api_key(cls)
                 if cls.models_needs_auth and not api_key:
                     raise MissingAuthError('Add a "api_key"')
-                if api_key is not None:
-                    headers["authorization"] = f"Bearer {api_key}"
-                response = requests.get(f"{api_base}/models", headers=headers, verify=cls.ssl)
+                response = requests.get(f"{api_base}/models", headers=cls.get_headers(False, api_key), verify=cls.ssl, timeout=timeout)
                 raise_for_status(response)
                 data = response.json()
-                data = data.get("data") if isinstance(data, dict) else data
+                data = data.get("data", data.get("models")) if isinstance(data, dict) else data
                 if (not cls.needs_auth or cls.models_needs_auth or api_key) and data:
                     cls.live += 1
                 cls.image_models = [model.get("name") if cls.use_model_names else model.get("id", model.get("name")) for model in data if model.get("image") or model.get("type") == "image" or model.get("supports_images")]
@@ -64,7 +61,7 @@ class OpenaiTemplate(AsyncGeneratorProvider, ProviderModelMixin, RaiseErrorMixin
                 if cls.fallback_models:
                     debug.error(e)
                     return cls.fallback_models
-                raise e
+                raise
         return cls.models
 
     @classmethod
@@ -89,7 +86,7 @@ class OpenaiTemplate(AsyncGeneratorProvider, ProviderModelMixin, RaiseErrorMixin
         headers: dict = None,
         impersonate: str = None,
         download_media: bool = True,
-        extra_parameters: list[str] = ["tools", "parallel_tool_calls", "tool_choice", "reasoning_effort", "logit_bias", "modalities", "audio", "stream_options"],
+        extra_parameters: list[str] = ["tools", "parallel_tool_calls", "tool_choice", "reasoning_effort", "logit_bias", "modalities", "audio", "stream_options", "include_reasoning", "response_format", "max_completion_tokens", "reasoning_effort", "search_settings"],
         extra_body: dict = None,
         **kwargs
     ) -> AsyncResult:
@@ -127,6 +124,8 @@ class OpenaiTemplate(AsyncGeneratorProvider, ProviderModelMixin, RaiseErrorMixin
                     yield ImageResponse([image["url"] for image in data["data"]], prompt)
                 return
 
+            if stream or stream is None:
+                kwargs.setdefault("stream_options", {"include_usage": True})
             extra_parameters = {key: kwargs[key] for key in extra_parameters if key in kwargs}
             if extra_body is None:
                 extra_body = {}
@@ -144,10 +143,11 @@ class OpenaiTemplate(AsyncGeneratorProvider, ProviderModelMixin, RaiseErrorMixin
                 **extra_body
             )
             if api_endpoint is None:
-                if api_base:
-                    api_endpoint = f"{api_base.rstrip('/')}/chat/completions"
                 if api_endpoint is None:
                     api_endpoint = cls.api_endpoint
+                if api_endpoint is None:
+                    api_endpoint = f"{api_base.rstrip('/')}/chat/completions"
+            yield JsonRequest.from_dict(data)
             async with session.post(api_endpoint, json=data, ssl=cls.ssl) as response:
                 async for chunk in read_response(response, stream, prompt, cls.get_dict(), download_media):
                     yield chunk
@@ -168,6 +168,7 @@ async def read_response(response: StreamResponse, stream: bool, prompt: str, pro
     content_type = response.headers.get("content-type", "text/event-stream" if stream else "application/json")
     if content_type.startswith("application/json"):
         data = await response.json()
+        yield JsonResponse.from_dict(data)
         OpenaiTemplate.raise_error(data, response.status)
         await raise_for_status(response)
         model = data.get("model")
@@ -204,6 +205,7 @@ async def read_response(response: StreamResponse, stream: bool, prompt: str, pro
         first = True
         model_returned = False
         async for data in sse_stream(response):
+            yield JsonResponse.from_dict(data)
             OpenaiTemplate.raise_error(data)
             model = data.get("model")
             if not model_returned and model:
