@@ -37,7 +37,8 @@ class RequestConfig(BaseModel):
     modalities: Optional[list[str]] = None
     audio: Optional[dict] = None
     response_format: Optional[dict] = None
-    download_media: bool = True
+    download_media: bool = False
+    raw: bool = False
     extra_body: Optional[dict] = None
 
 class ChatCompletionsConfig(RequestConfig):
