@@ -14,6 +14,9 @@ def get_config_dir() -> Path:
         return Path.home() / "Library" / "Application Support"
     return Path.home() / ".config"
 
+DEFAULT_PORT = 1337
+DEFAULT_TIMEOUT = 600
+DEFAULT_STREAM_TIMEOUT = 15
 
 PACKAGE_NAME = "g4f"
 CONFIG_DIR = get_config_dir() / PACKAGE_NAME
@@ -22,7 +25,7 @@ CUSTOM_COOKIES_DIR = "./har_and_cookies"
 ORGANIZATION = "gpt4free"
 GITHUB_REPOSITORY = f"xtekky/{ORGANIZATION}"
 STATIC_DOMAIN = f"{PACKAGE_NAME}.dev"
-STATIC_URL = f"https://{STATIC_DOMAIN}/"
+STATIC_URL = f"https://static.{STATIC_DOMAIN}/"
 DIST_DIR = f"./{STATIC_DOMAIN}/dist"
 DEFAULT_MODEL = "openai/gpt-oss-120b"
 JSDELIVR_URL = "https://cdn.jsdelivr.net/"
