@@ -21,7 +21,7 @@ from urllib.parse import quote_plus
 from hashlib import sha256
 
 try:
-    from PIL import Image 
+    from PIL import Image, UnidentifiedImageError
     has_pillow = True
 except ImportError:
     has_pillow = False
@@ -239,7 +239,7 @@ class Backend_Api(Api):
             cache_dir = Path(get_cookies_dir()) / ".usage"
             cache_file = cache_dir / f"{datetime.date.today()}.jsonl"
             cache_dir.mkdir(parents=True, exist_ok=True)
-            data = {**request.json, "user": request.headers.get("x-user", "unknown")}
+            data = {"user": request.headers.get("x-user", "unknown"), **request.json}
             with cache_file.open("a" if cache_file.exists() else "w") as f:
                 f.write(f"{json.dumps(data)}\n")
             return {}
@@ -356,11 +356,13 @@ class Backend_Api(Api):
                     if response.startswith("/media/"):
                         media_dir = get_media_dir()
                         filename = os.path.basename(response.split("?")[0])
-                        try:
-                            return send_from_directory(os.path.abspath(media_dir), filename)
-                        finally:
-                            if not cache_id:
+                        if not cache_id:
+                            try:
+                                return send_from_directory(os.path.abspath(media_dir), filename)
+                            finally:
                                 os.remove(os.path.join(media_dir, filename))
+                        else:
+                            return redirect(response)
                     elif response.startswith("https://") or response.startswith("http://"):
                         return redirect(response)
                 if do_filter:
@@ -452,7 +454,10 @@ class Backend_Api(Api):
                             image_size = {"width": width, "height": height}
                             thumbnail_dir = os.path.join(bucket_dir, "thumbnail")
                             os.makedirs(thumbnail_dir, exist_ok=True)
-                            process_image(image, save=os.path.join(thumbnail_dir, filename))
+                            width, height = process_image(image, save=os.path.join(thumbnail_dir, filename))
+                            image_size = {"width": width, "height": height}
+                        except UnidentifiedImageError:
+                            pass
                         except Exception as e:
                             logger.exception(e)
                     if result:
