@@ -27,6 +27,7 @@ from .Provider import (
     PollinationsAI,
     PollinationsImage,
     Qwen,
+    StringableInference,
     TeachAnything,
     Together,
     WeWordle,
@@ -154,15 +155,14 @@ default = Model(
     name = "",
     base_provider = "",
     best_provider = IterListProvider([
+        StringableInference,
         OIVSCodeSer0501,
         OIVSCodeSer2,
-        Blackbox,
         Copilot,
         DeepInfra,
         OperaAria,
         Startnest,
         GLM,
-        Kimi,
         PollinationsAI,
         Qwen,
         Together,
@@ -179,7 +179,7 @@ default_vision = VisionModel(
     name = "",
     base_provider = "",
     best_provider = IterListProvider([
-        Blackbox,
+        StringableInference,
         DeepInfra,
         OIVSCodeSer0501,
         OIVSCodeSer2,
@@ -200,20 +200,20 @@ default_vision = VisionModel(
 gpt_4 = Model(
     name          = 'gpt-4',
     base_provider = 'OpenAI',
-    best_provider = IterListProvider([Blackbox, Copilot, Yqcloud, WeWordle, OpenaiChat])
+    best_provider = IterListProvider([Copilot, Yqcloud, WeWordle, OpenaiChat])
 )
 
 # gpt-4o
 gpt_4o = VisionModel(
     name          = 'gpt-4o',
     base_provider = 'OpenAI',
-    best_provider = IterListProvider([Blackbox, OpenaiChat])
+    best_provider = IterListProvider([OpenaiChat])
 )
 
 gpt_4o_mini = Model(
     name          = 'gpt-4o-mini',
     base_provider = 'OpenAI',
-    best_provider = IterListProvider([Blackbox, Chatai, OIVSCodeSer2, Startnest, OpenaiChat])
+    best_provider = IterListProvider([Chatai, OIVSCodeSer2, Startnest, OpenaiChat])
 )
 
 gpt_4o_mini_audio = AudioModel(
@@ -271,19 +271,19 @@ o4_mini_high = Model(
 gpt_4_1 = Model(
     name          = 'gpt-4.1',
     base_provider = 'OpenAI',
-    best_provider = IterListProvider([PollinationsAI, OpenaiChat])
+    best_provider = IterListProvider([OpenaiChat])
 )
 
 gpt_4_1_mini = Model(
     name          = 'gpt-4.1-mini',
     base_provider = 'OpenAI',
-    best_provider = IterListProvider([Blackbox, OIVSCodeSer0501])
+    best_provider = IterListProvider([OpenaiChat])
 )
 
 gpt_4_1_nano = Model(
     name          = 'gpt-4.1-nano',
     base_provider = 'OpenAI',
-    best_provider = IterListProvider([Blackbox])
+    best_provider = IterListProvider([PollinationsAI])
 )
 
 gpt_4_5 = Model(
@@ -586,13 +586,6 @@ gemma_3n_e4b = Model(
     best_provider = Together
 )
 
-### Blackbox AI ###
-blackboxai = Model(
-    name = 'blackboxai',
-    base_provider = 'Blackbox AI',
-    best_provider = Blackbox
-)
-
 ### CohereForAI ###
 command_r = Model(
     name = 'command-r',
@@ -813,7 +806,7 @@ deepseek_v3_0324_turbo = Model(
 deepseek_r1_0528 = Model(
     name = 'deepseek-r1-0528',
     base_provider = 'DeepSeek',
-    best_provider = IterListProvider([DeepInfra, PollinationsAI])
+    best_provider = DeepInfra
 )
 
 deepseek_r1_0528_turbo = Model(
