@@ -21,7 +21,7 @@ from .response import BaseConversation, AuthResult
 from .helper import concat_chunks
 from ..cookies import get_cookies_dir
 from ..errors import ModelNotFoundError, ResponseError, MissingAuthError, NoValidHarFileError, PaymentRequiredError, CloudflareError
-from ..tools.run_tools import AuthManager
+from ..tools.auth import AuthManager
 from .. import debug
 
 SAFE_PARAMETERS = [
@@ -373,6 +373,7 @@ class ProviderModelMixin:
     audio_models: dict = {}
     last_model: str = None
     models_loaded: bool = False
+    models_tags: dict[str, list[str]] = None
 
     @classmethod
     def get_models(cls, api_key: str = None, **kwargs) -> list[str]:
