@@ -151,11 +151,37 @@ class JsonMixin:
 class RawResponse(ResponseType, JsonMixin):
     pass
 
+class ObjectMixin:
+    def __init__(self, **kwargs) -> None:
+        """Initialize with keyword arguments as attributes."""
+        for key, value in kwargs.items():
+            setattr(self, key, ObjectMixin.from_dict(value) if isinstance(value, dict) else [ObjectMixin.from_dict(v) if isinstance(v, dict) else v for v in value] if isinstance(value, list) else value)
+
+    def get_dict(self) -> Dict:
+        """Return a dictionary of non-private attributes."""
+        return {
+            key: value.get_dict() if isinstance(value, ObjectMixin) else [v.get_dict() if isinstance(v, ObjectMixin) else v for v in value] if isinstance(value, list) else value
+            for key, value in self.__dict__.items()
+            if not key.startswith("__")
+        }
+
+    @classmethod
+    def from_dict(cls, data: Dict) -> JsonMixin:
+        """Create an instance from a dictionary."""
+        return cls(**data)
+
+class JsonResponse(ResponseType, ObjectMixin):
+    def __str__(self) -> str:
+        return str(self.get_dict())
+
 class HiddenResponse(ResponseType):
     def __str__(self) -> str:
         """Hidden responses return an empty string."""
         return ""
 
+class JsonRequest(HiddenResponse, ObjectMixin):
+    pass
+
 class FinishReason(JsonMixin, HiddenResponse):
     def __init__(self, reason: str) -> None:
         """Initialize with a reason."""
@@ -177,6 +203,7 @@ class Usage(JsonMixin, HiddenResponse):
         completionTokens: int = None,
         input_tokens: int = None,
         output_tokens: int = None,
+        output_tokens_details: Dict = None,
         **kwargs
     ):
         if promptTokens is not None:
@@ -187,6 +214,9 @@ class Usage(JsonMixin, HiddenResponse):
             kwargs["prompt_tokens"] = input_tokens
         if output_tokens is not None:
             kwargs["completion_tokens"] = output_tokens
+        if output_tokens_details is not None:
+            for key, value in output_tokens_details.items():
+                kwargs[key] = value
         if "total_tokens" not in kwargs and "prompt_tokens" in kwargs and "completion_tokens" in kwargs:
             kwargs["total_tokens"] = kwargs["prompt_tokens"] + kwargs["completion_tokens"]
         return super().__init__(**kwargs)
@@ -204,10 +234,17 @@ class DebugResponse(HiddenResponse):
         """Initialize with a log message."""
         self.log = log
 
+class PlainTextResponse(HiddenResponse):
+    def __init__(self, text: str) -> None:
+        self.text = text
+
+class VariantResponse(HiddenResponse):
+    def __init__(self, text: str) -> None:
+        self.text = text
+
 class ContinueResponse(HiddenResponse):
-    def __init__(self, log: str) -> None:
-        """Initialize with a log message."""
-        self.log = log
+    def __init__(self, text: str) -> None:
+        self.text = text
 
 class Reasoning(ResponseType):
     def __init__(
@@ -266,7 +303,7 @@ class Sources(ResponseType):
         if not self.list:
             return ""
         return "\n\n\n\n" + ("\n>\n".join([
-            f"> [{idx}] {format_link(link['url'], link.get('title', None))}"
+            f"> [{idx}] {format_link(link['url'], link.get('title', link.get('name', None)))}"
             for idx, link in enumerate(self.list)
         ]))
 
@@ -376,8 +413,8 @@ class ImageResponse(MediaResponse):
         """Return images as markdown."""
         if self.get("width") and self.get("height"):
             return "\n".join([
-                f'<a href="{html.escape(url)}" data-width="{self.get("width")}" data-height="{self.get("height")}" data-source="{html.escape(self.get("source_url", ""))}">'
-                + f'<img src="{url.replace("/media/", "/thumbnail/")}" alt="{html.escape(" ".join(self.alt.split()))}"></a>'
+                f'<a href="{html.escape(url)}" data-src="{self.get("image", url)}" data-width="{self.get("width")}" data-height="{self.get("height")}" data-source="{html.escape(self.get("source_url", ""))}">'
+                + f'<img src="{self.get("thumbnail", url.replace("/media/", "/thumbnail/"))}" alt="{html.escape(self.alt)}" width="{html.escape(str(self.get("thumbnail_width", "")))}" height="{html.escape(str(self.get("thumbnail_height", "")))}"></a>'
                 for url in self.get_list()
             ])
         return format_images_markdown(self.urls, self.alt, self.get("preview"))
@@ -395,7 +432,7 @@ class VideoResponse(MediaResponse):
             return "\n".join(result)
         return "\n".join([f'<video src="{quote_url(video)}"></video>' for video in self.get_list()])
 
-class ImagePreview(ImageResponse, HiddenResponse):
+class ImagePreview(HiddenResponse, ImageResponse):
     pass
 
 class PreviewResponse(HiddenResponse):
@@ -405,12 +442,12 @@ class PreviewResponse(HiddenResponse):
 
     def to_string(self) -> str:
         """Return data as a string."""
-        return self.data
+        return "".join([str(item) for item in self.data]) if isinstance(self.data, list) else str(self.data)
 
 class Parameters(ResponseType, JsonMixin):
     def __str__(self) -> str:
         """Return an empty string."""
         return ""
 
-class ProviderInfo(JsonMixin, HiddenResponse):
+class ProviderInfo(HiddenResponse, JsonMixin):
     pass
