@@ -8,6 +8,7 @@ from urllib.parse import urlparse
 from typing import Iterator, AsyncIterator
 from http.cookies import Morsel
 from pathlib import Path
+from contextlib import asynccontextmanager
 import asyncio
 try:
     from curl_cffi.requests import Session, Response
@@ -113,7 +114,6 @@ async def get_args_from_nodriver(
             await callback(page)
         for c in await page.send(nodriver.cdp.network.get_cookies([url])):
             cookies[c.name] = c.value
-        await page.close()
         stop_browser()
         return {
             "impersonate": "chrome",
@@ -191,6 +191,8 @@ async def get_nodriver(
         debug.log(f"Open nodriver with user_dir: {user_data_dir}")
     try:
         browser_args = ["--no-sandbox"]
+        if BrowserConfig.port:
+            browser_executable_path = "/bin/google-chrome"
         browser = await nodriver.start(
             user_data_dir=user_data_dir,
             browser_args=[*browser_args, f"--proxy-server={proxy}"] if proxy else browser_args,
@@ -219,7 +221,13 @@ async def get_nodriver(
     BrowserConfig.stop_browser = on_stop
     return browser, on_stop
 
-async def sse_stream(iter_lines: Iterator[bytes]) -> AsyncIterator[dict]:
+@asynccontextmanager
+async def get_nodriver_session(**kwargs):
+    browser, stop_browser = await get_nodriver(**kwargs)
+    yield browser
+    stop_browser()
+
+async def sse_stream(iter_lines: AsyncIterator[bytes]) -> AsyncIterator[dict]:
     if hasattr(iter_lines, "content"):
         iter_lines = iter_lines.content
     elif hasattr(iter_lines, "iter_lines"):
