@@ -69,12 +69,12 @@ try:
 except ImportError:
     has_beautifulsoup4 = False
 try:
-    from markitdown import MarkItDown
+    from g4f.integration.markitdown import MarkItDown
     has_markitdown = True
 except ImportError:
     has_markitdown = False
 
-from .web_search import scrape_text
+from .fetch_and_scrape import scrape_text
 from ..files import secure_filename, get_bucket_dir
 from ..image import is_allowed_extension
 from ..requests.aiohttp import get_connector
@@ -434,7 +434,7 @@ async def download_urls(
             text_content = None
             if has_markitdown:
                 try:
-                    text_content = md.convert(url).text_content
+                    text_content = md.convert_url(url).text_content
                     if text_content:
                         filename = get_filename_from_url(url)
                         target = bucket_dir / filename
